@@ -28,7 +28,7 @@ def get_services_list(ctx, args, incomplete):
     'graphdb', 'virtuoso', 'tmp-virtuoso', 'blazegraph', 'allegrograph', 'anzograph', 'fuseki',
     'into-the-graph', 'ldf-server', 'comunica', 'notebook',
     'api', 'drill', 'postgres', 'proxy', 'filebrowser', 'rmlstreamer', 'rmltask',
-    'neo4j' ])
+    'limes-server', 'neo4j' ])
 def get_datasets_list(ctx, args, incomplete):
     return filter(lambda x: x.startswith(incomplete), os.listdir("./datasets"))
 def get_workflows_list(ctx, args, incomplete):
@@ -102,9 +102,9 @@ def init(ctx, projectname):
         os.system('git submodule add --recursive https://github.com/MaastrichtU-IDS/d2s-cwl-workflows.git')
 
     # Create workspace directories and chmod 777
-    listToCreate = ["input", "output", "import", "output/tmp-outdir", 
+    listToCreate = ["input", "output", "import", "output/tmp-outdir", "resources",
         "logs", "dumps/rdf/releases/1", 'dumps/hdt', 'virtuoso', 'tmp-virtuoso']
-    click.echo(click.style('[d2s]', bold=True) + ' Creating following folders in workspace: ' + ", ".join(listToUpdate))
+    click.echo(click.style('[d2s]', bold=True) + ' Creating following folders in workspace: ' + ", ".join(listToCreate))
     for fileToCreate in listToCreate:
         os.makedirs('workspace/' + fileToCreate, exist_ok=True)
     chmod777('workspace')
@@ -117,11 +117,11 @@ def init(ctx, projectname):
     # Get RMLStreamer from home dir to qvoid download each time
     user_home_dir = str(Path.home())
     if os.path.exists(user_home_dir + '/RMLStreamer.jar'):
-        shutil.copyfile(user_home_dir + '/RMLStreamer.jar', 'workspace/RMLStreamer.jar')
+        shutil.copyfile(user_home_dir + '/RMLStreamer.jar', 'workspace/resources/RMLStreamer.jar')
     else:
         click.echo(click.style('[d2s]', bold=True) + ' Downloading RMLStreamer.jar... [80M]')
-        urllib.request.urlretrieve ("https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar", "workspace/RMLStreamer.jar")
-    chmod777('workspace/RMLStreamer.jar')
+        urllib.request.urlretrieve ("https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar", "workspace/resources/RMLStreamer.jar")
+    chmod777('workspace/resources/RMLStreamer.jar')
 
     click.echo()
     # Copy GraphDB zip file to the right folder in d2s-cwl-workflows
@@ -324,16 +324,31 @@ def download(datasets):
 @click.option(
     '--detached/--watch', default=True, 
     help='Run in detached mode or watch workflow')
+@click.option(
+    '--openshift/--local', default=False, 
+    help='Run RML Streamer on OpenShift')
 @click.option(
     '-p', '--parallelism', default='8',
     help='Run in parallel, depends on Task Slots availables')
-def rml(dataset, detached, mapper, parallelism):
+def rml(dataset, detached, mapper, openshift, parallelism):
     """Run RML Streamer"""
     if (detached):
         detached_arg = '-d'
     else:
         detached_arg = '-it'
 
+    if openshift:
+        # Ask if need to copy file on OpenShift cluster
+        click.echo(click.style('[d2s]', bold=True) + ' Running RMLStreamer on OpenShift. Make sure your logged in and in the right project.')
+        flink_manager_pod = os.popen('oc get pod --selector app=flink --selector component=jobmanager --no-headers -o=custom-columns=NAME:.metadata.name').read().strip()
+        click.echo(click.style('[d2s]', bold=True) + ' ID of the Apache Flink pod used: ' 
+            + click.style(flink_manager_pod, bold=True))
+        if click.confirm(click.style('[?]', bold=True) + ' It is required to copy (rsync) the workspace to the OpenShift cluster the first time your run it. Do you want to do it?'):
+            # rsync input files and mapping files
+            os.system('oc exec ' + flink_manager_pod + ' -- mkdir -p /mnt/workspace/import')
+            os.system('oc rsync workspace/input ' + flink_manager_pod + ':/mnt/workspace/')
+            os.system('oc rsync datasets ' + flink_manager_pod + ':/mnt/')
+
     for file in os.listdir('./datasets/' + dataset + '/mapping'):
      mapping_filename = os.fsdecode(file)
      if mapping_filename.endswith(".rml.ttl"): 
@@ -343,18 +358,27 @@ def rml(dataset, detached, mapper, parallelism):
             + click.style(mapping_filepath, bold=True))
 
         output_filename = 'rml'
-        if mapper:
-            output_filename = 'rmlmapper-' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt'
-            rml_cmd = 'docker run ' + detached_arg + ' -v ' + getCurrentDir() + '/workspace:/mnt/workspace -v ' + getCurrentDir() + '/datasets:/mnt/datasets umids/rmlmapper:4.7.0 -m /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' -o /mnt/workspace/import/' + output_filename
+        # Now build the command to run RML processor
+        if openshift:
+            # Run RMLStreamer in an OpenShift cluster
+            output_filename = 'openshift-rmlstreamer-' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt'
+            rml_cmd = 'oc exec ' + flink_manager_pod + ' -- /opt/flink/bin/flink run -c io.rml.framework.Main /opt/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/import/' + output_filename + ' --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '"'
         else:
-            # Use RMLStreamer
-            output_filename = 'rmlstreamer-' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt'
-            rml_cmd = 'docker exec ' + detached_arg + ' d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -c io.rml.framework.Main /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/import/' + output_filename + ' --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '"'
-            click.echo(click.style('[d2s]', bold=True) + ' Check the jobs running at ' 
-                + click.style('http://localhost:8078/#/job/running', bold=True))
-            ## Try parallelism:
-            # rml_cmd = 'docker exec ' + detached_arg + ' d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -p ' + parallelism + ' -c io.rml.framework.Main /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/import/' + output_filename + ' --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '" --parallelism ' + parallelism + ' --enable-local-parallel'
-    
+            # Run locally
+            if mapper:
+                # Run rmlmapper docker image
+                output_filename = 'rmlmapper-' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt'
+                rml_cmd = 'docker run ' + detached_arg + ' -v ' + getCurrentDir() + '/workspace:/mnt/workspace -v ' + getCurrentDir() + '/datasets:/mnt/datasets umids/rmlmapper:4.7.0 -m /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' -o /mnt/workspace/import/' + output_filename
+            else:
+                # Run RMLStreamer in running Apache Flink
+                output_filename = 'rmlstreamer-' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt'
+                rml_cmd = 'docker exec ' + detached_arg + ' d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -c io.rml.framework.Main /opt/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/import/' + output_filename + ' --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '"'
+                click.echo(click.style('[d2s]', bold=True) + ' Check the jobs running at ' 
+                    + click.style('http://localhost:8078/#/job/running', bold=True))
+                ## Try parallelism:
+                # rml_cmd = 'docker exec ' + detached_arg + ' d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -p ' + parallelism + ' -c io.rml.framework.Main /mnt/workspace/resources/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/import/' + output_filename + ' --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '" --parallelism ' + parallelism + ' --enable-local-parallel'
+        
+        # Run RML processor
         os.system(rml_cmd)
         # TODO: store logs in a file and get the run time. Use subprocess.call
         # with open('workspace/logs/' + output_filename + '.log', 'a') as output:
@@ -366,9 +390,9 @@ def rml(dataset, detached, mapper, parallelism):
         click.secho('workspace/import/' + output_filename, bold=True)
     
     # Try parallelism:
-    # rmlstreamer_cmd = 'docker exec -d d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -p 4 /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '" --enable-local-parallel'
+    # rmlstreamer_cmd = 'docker exec -d d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -p 4 /mnt/workspace/resources/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '" --enable-local-parallel'
     # Try to use docker-compose, but exec dont resolve from the -f file
-    # rmlstreamer_cmd = docker_compose_cmd + 'exec -d rmlstreamer /opt/flink/bin/flink run /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '"'
+    # rmlstreamer_cmd = docker_compose_cmd + 'exec -d rmlstreamer /opt/flink/bin/flink run /mnt/workspace/resources/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '"'
     # print(rmlstreamer_cmd)
 
 @cli.command()
